@@ -32,6 +32,8 @@
 
 (declare-function subed-subtitle-id "subed-common" ())
 (declare-function subed-subtitle-msecs-start "subed-common" (&optional id))
+(declare-function subed-subtitle-msecs-stop "subed-common" (&optional id))
+
 (defvar subed-mpv-frame-step-map)
 
 (defvar-local subed-mpv-is-playing nil
@@ -74,18 +76,21 @@
 
 ;;; Server (mpv process that provides an IPC socket)
 
-(defun subed-mpv--socket ()
+(defun subed-mpv--socket (&optional skip-create)
   "Path to mpv's RPC socket for a particular buffer.
-See also `subed-mpv-socket-dir'."
-  (unless (file-exists-p subed-mpv-socket-dir)
+See also `subed-mpv-socket-dir'.
+If SKIP-CREATE is non-nil, don't create it if it doesn't exist."
+  (unless (or skip-create (file-exists-p subed-mpv-socket-dir))
     (condition-case err
         (make-directory subed-mpv-socket-dir :create-parents)
       (file-error
-       (error "%s" (mapconcat #'identity (cdr err) ": ")))))
-  (expand-file-name
-   (format "subed-%s"
-           (md5 (subed--buffer-file-name)))
-   subed-mpv-socket-dir))
+       (error "%s" (mapconcat #'identity (cdr err) ": "))
+       nil)))
+  (when (file-exists-p subed-mpv-socket-dir)
+    (expand-file-name
+     (format "subed-%s"
+             (md5 (subed--buffer-file-name)))
+     subed-mpv-socket-dir)))
 
 (defun subed-mpv--server-start (&rest args)
   "Run mpv in JSON IPC mode.
@@ -115,10 +120,11 @@ Pass ARGS as command line arguments.  \"--idle\" and
   (when (and subed-mpv--server-proc (process-live-p subed-mpv--server-proc))
     (delete-process subed-mpv--server-proc)
     (subed-debug "Killed mpv process"))
-  (let ((socket-file (subed-mpv--socket)))
-    (when (file-exists-p socket-file)
-      (subed-debug "Removing IPC socket: %s" socket-file)
-      (ignore-errors (delete-file socket-file))))
+  (ignore-errors
+    (let ((socket-file (subed-mpv--socket t)))
+      (when (file-exists-p socket-file)
+        (subed-debug "Removing IPC socket: %s" socket-file)
+        (ignore-errors (delete-file socket-file)))))
   (setq subed-mpv--server-proc nil))
 
 (defun subed-mpv--server-started-p ()
@@ -148,7 +154,7 @@ again with (cdr DELAYS) as arguments."
                                     :family 'local
                                     :service (subed-mpv--socket)
                                     :coding '(utf-8 . utf-8)
-					                :buffer (subed-mpv--client-buffer)
+                          :buffer (subed-mpv--client-buffer)
                                     :filter #'subed-mpv--client-filter
                                     :noquery t
                                     :nowait t)))
@@ -183,7 +189,7 @@ again with (cdr DELAYS) as arguments."
 
 (defun subed-mpv--client-connected-p ()
   "Whether the server connection has been established and tested successfully."
-  (if subed-mpv--client-proc t nil))
+  (process-live-p subed-mpv--client-proc))
 
 (defun subed-mpv--client-send (cmd)
   "Send JSON IPC command.
@@ -218,28 +224,28 @@ string."
   (when (buffer-live-p (process-buffer proc))
     (let ((orig-buffer (current-buffer)))
       (when (derived-mode-p 'subed-mode)
-	      (with-current-buffer (process-buffer proc)
+        (with-current-buffer (process-buffer proc)
           ;; Insert new response where previous response ended
-	        (let* ((proc-mark (process-mark proc))
+          (let* ((proc-mark (process-mark proc))
                  (moving (= (point) proc-mark)))
-		        (save-excursion
-		          (goto-char proc-mark)
-		          (insert response)
-		          (set-marker proc-mark (point)))
-		        (if moving (goto-char proc-mark)))
-	        ;; Process and remove all complete lines of JSON (lines are complete if
-	        ;; they end with \n)
-	        (let ((p0 (point-min)))
-		        (while (progn (goto-char p0)
+            (save-excursion
+              (goto-char proc-mark)
+              (insert response)
+              (set-marker proc-mark (point)))
+            (if moving (goto-char proc-mark)))
+          ;; Process and remove all complete lines of JSON (lines are complete if
+          ;; they end with \n)
+          (let ((p0 (point-min)))
+            (while (progn (goto-char p0)
                           (end-of-line)
-			                    (equal (following-char) ?\n))
-		          (let* ((p1 (point))
-			               (line (buffer-substring p0 p1)))
-			          (delete-region p0 (+ p1 1))
+                          (equal (following-char) ?\n))
+              (let* ((p1 (point))
+                     (line (buffer-substring p0 p1)))
+                (delete-region p0 (+ p1 1))
                 ;; Return context to the subtitle file buffer because we're using
                 ;; buffer-local variables to store player state.
                 (with-current-buffer orig-buffer
-			            (subed-mpv--client-handle-json line))))))))))
+                  (subed-mpv--client-handle-json line))))))))))
 
 (defun subed-mpv--client-handle-json (json-string)
   "Process server response JSON-STRING."
@@ -313,10 +319,13 @@ See \"List of events\" in mpv(1)."
 
 (defun subed-mpv-seek (msec)
   "Move playback position MSEC milliseconds relative to current position."
+  (interactive "NOffset in ms: ")
   (subed-mpv--client-send `(seek ,(/ msec 1000.0) relative+exact)))
 
 (defun subed-mpv-jump (msec)
   "Move playback position to absolute position MSEC milliseconds."
+  (interactive "MTimestamp or msecs: ")
+  (setq msec (subed-to-msecs msec))
   (subed-mpv--client-send `(seek ,(/ msec 1000.0) absolute+exact)))
 
 (defun subed-mpv-jump-to-current-subtitle ()
@@ -327,21 +336,82 @@ See \"List of events\" in mpv(1)."
       (subed-debug "Seeking player to focused subtitle: %S" cur-sub-start)
       (subed-mpv-jump cur-sub-start))))
 
+(defun subed-mpv-jump-to-current-subtitle-near-end ()
+  "Move playback position to near the end of the current subtitle."
+  (interactive)
+  (let ((cur-sub-end (subed-subtitle-msecs-stop)))
+    (when cur-sub-end
+      (setq cur-sub-end (- cur-sub-end subed-sample-msecs))
+      (subed-debug "Seeking player to end of focused subtitle: %S" cur-sub-end)
+      (subed-mpv-jump cur-sub-end))))
+
+(defun subed-mpv-jump-to-before-current-subtitle ()
+  "Move playback position before the current subtitle."
+  (interactive)
+  (let ((cur-sub-start (subed-subtitle-msecs-start)))
+    (when cur-sub-start
+      (setq cur-sub-start (- cur-sub-start subed-sample-msecs))
+      (subed-debug "Seeking player to before focused subtitle: %S" cur-sub-start)
+      (subed-mpv-jump cur-sub-start))))
+
+(defun subed-mpv-jump-to-end-of-current-subtitle ()
+  "Move playback position to the end of current subtitle."
+  (interactive)
+  (let ((cur-sub-end (subed-subtitle-msecs-stop)))
+    (when cur-sub-end
+      (subed-debug "Seeking player to end of focused subtitle: %S" cur-sub-end)
+      (subed-mpv-jump cur-sub-end))))
+
+(defun subed-mpv-back-large-step ()
+  "Move back one large step."
+  (interactive)
+  (subed-mpv--client-send `(seek ,(- 0 subed-mpv-large-step-seconds) relative+exact)))
+
+(defun subed-mpv-back-small-step ()
+  "Move back one small step."
+  (interactive)
+  (subed-mpv--client-send `(seek ,(- 0 subed-mpv-small-step-seconds) relative+exact)))
+
+(defun subed-mpv-small-step ()
+  "Move forward one small step."
+  (interactive)
+  (subed-mpv--client-send `(seek ,subed-mpv-small-step-seconds relative+exact)))
+
+(defun subed-mpv-large-step ()
+  "Move forward one large step."
+  (interactive)
+  (subed-mpv--client-send `(seek ,subed-mpv-large-step-seconds relative+exact)))
+
+(defun subed-mpv-undo-seek ()
+  "Undo a seek."
+  (interactive)
+  (subed-mpv--client-send `(revert-seek)))
+
 (defun subed-mpv-frame-step ()
   "Step one frame forward.
 Set up keybindings so that repeatedly pressing `,' and `.' moves
 by frames until any other key is pressed."
   (interactive)
-  (subed-mpv--client-send `(frame-step))
-  (set-transient-map subed-mpv-frame-step-map))
+  (subed-mpv--client-send `(frame-step)))
 
 (defun subed-mpv-frame-back-step ()
   "Step one frame backward.
 Set up keybindings so that repeatedly pressing `,' and `.' moves
 by frames until any other key is pressed."
   (interactive)
-  (subed-mpv--client-send `(frame-back-step))
-  (set-transient-map subed-mpv-frame-step-map))
+  (subed-mpv--client-send `(frame-back-step)))
+
+(defun subed-mpv-backward-subtitle-and-jump ()
+  "Go to the previous subtitle and jump to its beginning."
+  (interactive)
+  (subed-backward-subtitle-text)
+  (subed-mpv-jump-to-current-subtitle))
+
+(defun subed-mpv-forward-subtitle-and-jump ()
+  "Go to the next subtitle and jump to its beginning."
+  (interactive)
+  (subed-forward-subtitle-text)
+  (subed-mpv-jump-to-current-subtitle))
 
 (defun subed-mpv-add-subtitles (file)
   "Load FILE as subtitles in mpv."
@@ -351,6 +421,70 @@ by frames until any other key is pressed."
   "Reload subtitle file from disk."
   (subed-mpv--client-send '(sub-reload)))
 
+(defun subed-mpv-screenshot (&optional file flags)
+  "Take a screenshot without subtitles.
+Save to FILE if specified.
+Use \\[universal-argument] to save to a specific file.
+Returns the filename."
+  (setq flags (or flags 'video))
+  (interactive (list (if current-prefix-arg (read-file-name "Filename: "))))
+  (if file
+      (subed-mpv--client-send `(screenshot-to-file ,file ,flags))
+    (let ((old-latest
+           (car (sort (seq-remove #'file-directory-p
+                                  (directory-files default-directory 'full "mpv-shot[0-9]+\\.jpg" t))
+                      #'file-newer-than-file-p))))
+      (subed-mpv--client-send `(screenshot ,flags))
+      (while (string=
+              (car (sort (seq-remove #'file-directory-p
+                                     (directory-files default-directory 'full "mpv-shot[0-9]+\\.jpg" t))
+                         #'file-newer-than-file-p))
+              old-latest)
+        (sit-for 0.5)
+        (setq file (car (sort (seq-remove #'file-directory-p
+                                          (directory-files default-directory 'full "mpv-shot[0-9]+\\.jpg" t))
+                              #'file-newer-than-file-p))))))
+  (kill-new file)
+  (message "Copied %s" file)
+  file)
+
+(defun subed-mpv-screenshot-with-subtitles (&optional file)
+  "Take a screenshot including subtitles.
+Save to FILE if specified.
+Use \\[universal-argument] to save to a specific file.
+Returns the filename."
+  (interactive (list (if current-prefix-arg (read-file-name "Filename: "))))
+  (subed-mpv-screenshot file 'subtitles))
+
+(defun subed-mpv-copy-position-as-timestamp ()
+  "Copy current playback position as a timestamp."
+  (interactive)
+  (if subed-mpv-playback-position
+      (progn
+        (let ((pos (subed-msecs-to-timestamp subed-mpv-playback-position)))
+          (kill-new pos)
+          (message "Copied %s" pos)))
+    (error "No playback position yet")))
+
+(defun subed-mpv-copy-position-as-seconds ()
+  "Copy current playback position as seconds.
+This might be helpful for ffmpeg."
+  (interactive)
+  (if subed-mpv-playback-position
+      (let ((pos (number-to-string (/ subed-mpv-playback-position 1000.0))))
+        (kill-new pos)
+        (message "Copied %s" pos))
+    (error "No playback position yet")))
+
+(defun subed-mpv-copy-position-as-msecs ()
+  "Copy current playback position as msecs."
+  (interactive)
+  (if subed-mpv-playback-position
+      (let ((pos (number-to-string subed-mpv-playback-position)))
+        (kill-new pos)
+        (message "Copied %s" pos))
+    (error "No playback position yet")))
+
 (define-obsolete-function-alias 'subed-mpv--is-video-file-p 'subed-mpv--is-media-file-p "1.20")
 (defun subed-mpv--is-media-file-p (filename)
   "Return non-nil if FILENAME is a media file.
@@ -387,6 +521,10 @@ hosting providers."
   (setq subed-mpv-media-file url)
   (subed-mpv--play url))
 
+(defvar subed-mpv-play-from-file-hook nil
+  "Functions to run after a media file is loaded.
+Called with FILE as argument.")
+
 (defun subed-mpv-play-from-file (file)
   "Open FILE in mpv.
 
@@ -394,7 +532,10 @@ Files are expected to have any of the extensions listed in
 `subed-video-extensions' or `subed-audio-extensions'."
   (interactive (list (read-file-name "Find media: " nil nil t nil #'subed-mpv--is-media-file-p)))
   (setq subed-mpv-media-file (expand-file-name file))
-  (subed-mpv--play (expand-file-name file)))
+  (subed-clear-file-duration-ms-cache)
+  (subed-mpv--play (expand-file-name file))
+  (run-hook-with-args subed-mpv-play-from-file-hook))
+
 (define-obsolete-function-alias 'subed-mpv-find-video 'subed-mpv-play-from-file "1.20")
 
 (defun subed-mpv--add-subtitle-after-first-save ()
@@ -415,5 +556,11 @@ time the buffer is saved."
   (subed-mpv--client-disconnect)
   (subed-mpv--server-stop))
 
+(defun subed-mpv-control ()
+  "Use keyboard shortcuts to control MPV.
+See `subed-mpv-control-map'."
+  (interactive)
+  (set-transient-map subed-mpv-control-map t))
+
 (provide 'subed-mpv)
 ;;; subed-mpv.el ends here
