@@ -43,8 +43,8 @@
 
 ;;; Parsing
 
-(defconst subed-vtt--regexp-timestamp "\\(\\([0-9]+\\):\\)?\\([0-9]+\\):\\([0-9]+\\)\\.\\([0-9]+\\)")
-(defconst subed-vtt--regexp-separator "\\(?:[[:blank:]]*\n\\)+\\(?:NOTE[ \n]\\(?:.+?\n\\)+\n\\)*\n")
+(defconst subed-vtt--regexp-timestamp "\\(\\([0-9]+\\):\\)?\\([0-9]+\\):\\([0-9]+\\)\\(?:\\.\\([0-9]+\\)\\)?")
+(defconst subed-vtt--regexp-separator "\\(?:[[:blank:]]*\n\n+NOTE[ \n]\\(?:.+?\n\\)+\\)*\n\n+")
 (defconst subed-vtt--regexp-identifier
   ;; According to https://developer.mozilla.org/en-US/docs/Web/API/WebVTT_API
   ;; Cues can start with an identifier which is a non empty line that does
@@ -59,7 +59,7 @@ Use the format-specific function for MAJOR-MODE."
     (let ((hours (string-to-number (or (match-string 2 time-string) "0")))
           (mins  (string-to-number (match-string 3 time-string)))
           (secs  (string-to-number (match-string 4 time-string)))
-          (msecs (string-to-number (subed--right-pad (match-string 5 time-string) 3 ?0))))
+          (msecs (string-to-number (subed--right-pad (or (match-string 5 time-string) "0")  3 ?0))))
       (+ (* (truncate hours) 3600000)
          (* (truncate mins) 60000)
          (* (truncate secs) 1000)
@@ -94,18 +94,6 @@ format-specific function for MAJOR-MODE."
         (unless (subed-forward-subtitle-id)
           (throw 'subtitle-id nil))))))
 
-(cl-defmethod subed--subtitle-comment (&context (major-mode subed-vtt-mode) &optional sub-id)
-  "Return the comment or comments before the current subtitle.
-If SUB-ID is specified, jump to that subtitle first.
-Use the format-specific function for MAJOR-MODE."
-  (save-excursion
-    (subed-jump-to-subtitle-id sub-id)
-    (let ((sub-start-point (point)))
-      (or (subed-backward-subtitle-end)
-          (goto-char (point-min)))
-      (when (re-search-forward "^\\(NOTE\\(.*\n\\)+\n+\\)" sub-start-point t)
-        (match-string 0)))))
-
 ;;; Traversing
 
 (cl-defmethod subed--jump-to-subtitle-id (&context (major-mode subed-vtt-mode) &optional sub-id)
@@ -119,25 +107,35 @@ Use the format-specific function for MAJOR-MODE."
         ;; Look for a line that contains the timestamp, preceded by one or more
         ;; blank lines or the beginning of the buffer.
         (let* ((regex (concat "\\(" subed--regexp-separator "\\|\\`\\)\\("
-                              (regexp-quote sub-id) "\\)")))
+                              (regexp-quote sub-id) "\\)[ \n]")))
           (goto-char (point-min))
           (setq found (re-search-forward regex nil t))
           (if found
               (goto-char (match-beginning 2))
             (goto-char orig-point)
             nil))
-
       ;; Find two or more blank lines or the beginning of the buffer, followed
       ;; by an optional line and a timestamp.
       (or (and (re-search-backward subed--regexp-separator nil t)
                (goto-char (match-end 0)))
           (goto-char (point-min)))
       (cond
-       ((looking-at (concat "\\(" subed-vtt--regexp-timestamp "\\) *--> *" subed-vtt--regexp-timestamp " *\n"))
+       ((looking-at (concat "\\(" subed-vtt--regexp-timestamp "\\) *--> *" subed-vtt--regexp-timestamp "\\(?:\s+.+\\)?\n"))
         ;; no ID, use the timestamp
         (point))
-       ((looking-at (concat "\\(.*\\)\n" subed-vtt--regexp-timestamp " *--> *" subed-vtt--regexp-timestamp " *\n"))
+       ((looking-at (concat "\\(.*\\)\n" subed-vtt--regexp-timestamp " *--> *" subed-vtt--regexp-timestamp "\\(?:\s+.+\\)?\n"))
         (point))
+       ((looking-at "^NOTE[ \n]")
+        ;; At a subtitle's comment; scan forward for the timestamp
+        (if (re-search-forward
+             (concat
+              subed--regexp-separator
+              (concat "\\(.*\n\\)?\\(" subed-vtt--regexp-timestamp " *--> *" subed-vtt--regexp-timestamp "\\)\\(?:\s+.+\\)?*\n")) nil t)
+            (progn
+              (goto-char (or (match-beginning 1) (match-beginning 2) (point)))
+              (point))
+          (goto-char orig-point)
+          nil))
        (t
         (goto-char orig-point)
         nil)))))
@@ -180,10 +178,7 @@ Return point or nil if point did not change or if no subtitle end
 can be found.  Use the format-specific function for MAJOR-MODE."
   (let ((orig-point (point)))
     (subed-jump-to-subtitle-text sub-id)
-    ;; Look for next separator or end of buffer.  We can't use
-    ;; `subed-vtt--regexp-separator' here because if subtitle text is empty,
-    ;; it may be the only empty line in the separator, i.e. there's only one
-    ;; "\n".
+    ;; Look for next separator or end of buffer.
     (let ((regex subed-vtt--regexp-separator))
       (if (eolp)
           (unless (= (point) orig-point)
@@ -200,11 +195,13 @@ can be found.  Use the format-specific function for MAJOR-MODE."
 Return point or nil if there is no next subtitle.  Use the
 format-specific function for MAJOR-MODE."
   (let ((orig-point (point)))
-    (if (and
-         (re-search-forward subed--regexp-separator nil t)
-         (re-search-forward (concat "^" subed--regexp-timestamp
-                                    " *--> *" subed--regexp-timestamp)
-                            nil t))
+    (subed-jump-to-subtitle-end)
+    (when (and (bolp) (> (point) (point-min))) (forward-char -1))
+    (if (re-search-forward (concat subed--regexp-separator
+                                   "\\(.*?\n\\)?"
+                                   subed--regexp-timestamp
+                                   " *--> *" subed--regexp-timestamp)
+                           nil t)
         (or (subed-jump-to-subtitle-id)
             (progn
               (goto-char orig-point)
@@ -217,13 +214,17 @@ format-specific function for MAJOR-MODE."
 Return point or nil if there is no previous subtitle.  Use the
 format-specific function for MAJOR-MODE."
   (let ((orig-point (point)))
-    (when (subed-jump-to-subtitle-id)
-      (or
-       (catch 'found
-         (while (re-search-backward subed--regexp-separator nil t)
-           (when (subed-jump-to-subtitle-id)
-             (throw 'found (point)))))
-       (progn (goto-char orig-point) nil)))))
+    (subed-jump-to-subtitle-id)
+    (or
+     (catch 'found
+       (while (re-search-backward
+               (concat "^\\(.*?\\n\\)?"
+                       subed--regexp-timestamp
+                       " *--> *" subed--regexp-timestamp)
+               nil t)
+         (when (subed-jump-to-subtitle-id)
+           (throw 'found (point)))))
+     (progn (goto-char orig-point) nil))))
 
 ;;; Manipulation
 
@@ -236,9 +237,51 @@ Make sure COMMENT ends with a blank line."
                        (concat comment
                                (if (string-match "\n\n\\'" comment)
                                    "" "\n\n")))
-				 comment)
+         comment)
         ((string-match "\n" comment) (concat "NOTE\n" comment "\n\n"))
-        (t (concat "NOTE " comment))))
+        (t (concat "NOTE " comment "\n\n"))))
+
+(cl-defmethod subed--jump-to-subtitle-comment (&context (major-mode subed-vtt-mode)
+                                                        &optional sub-id)
+  "Move point to subtitle's comment.
+If SUB-ID is not given, use subtitle on point.
+Return point, or nil if no comment could be found.
+Use the format-specific function for MAJOR-MODE."
+  (let ((pos (point)))
+    (if (and (subed-jump-to-subtitle-id sub-id)
+             (re-search-backward "^NOTE"
+                                 (or (save-excursion (subed-backward-subtitle-end)) (point-min)) t))
+        (progn (goto-char (match-beginning 0))
+               (point))
+      (goto-char pos)
+      nil)))
+
+(cl-defmethod subed--subtitle-comment (&context (major-mode subed-vtt-mode)
+                                                &optional sub-id)
+  "Return subtitle comment or nil if none.
+If SUB-ID is not given, use the subtitle on point.
+Use the format-specific function for MAJOR-MODE."
+  (save-excursion
+    (let ((comment-start (subed-jump-to-subtitle-comment sub-id)))
+      (if comment-start
+          (string-trim
+           (replace-regexp-in-string "^NOTE[ \n]+" ""
+                                     (buffer-substring comment-start (subed-jump-to-subtitle-id sub-id))))
+        nil))))
+
+(cl-defmethod subed--set-subtitle-comment (comment &context (major-mode subed-vtt-mode)
+                                                   &optional sub-id)
+  "Set the current subtitle's comment to COMMENT.
+If COMMENT is nil or the empty string, remove the comment.
+If SUB-ID is not given, use the subtitle on point.
+Use the format-specific function for MAJOR-MODE."
+  (let ((comment-start (subed-jump-to-subtitle-comment sub-id)))
+    ;; remove previous comment
+    (if comment-start
+        (delete-region comment-start (subed-jump-to-subtitle-id sub-id))
+      (subed-jump-to-subtitle-id sub-id))
+    (when (and comment (not (string= comment "")))
+      (insert (subed-vtt--format-comment comment)))))
 
 (cl-defmethod subed--make-subtitle (&context (major-mode subed-vtt-mode)
                                              &optional _ start stop text comment)
@@ -268,7 +311,7 @@ TEXT defaults to an empty string.
 
 Move point to the text of the inserted subtitle.  Return new
 point.  Use the format-specific function for MAJOR-MODE."
-  (subed-jump-to-subtitle-id)
+  (or (subed-jump-to-subtitle-comment) (subed-jump-to-subtitle-id))
   (insert (subed-make-subtitle id start stop text comment))
   (when (looking-at (concat "\\([[:space:]]*\\|^\\)" subed--regexp-timestamp))
     (insert "\n"))
@@ -284,7 +327,7 @@ TEXT defaults to an empty string.
 
 Move point to the text of the inserted subtitle.  Return new
 point.  Use the format-specific function for MAJOR-MODE."
-  (unless (subed-forward-subtitle-id)
+  (unless (or (subed-forward-subtitle-comment) (subed-forward-subtitle-id))
     ;; Point is on last subtitle or buffer is empty
     (subed-jump-to-subtitle-end)
     (when (looking-at "[[:space:]]+")
@@ -314,8 +357,10 @@ Use the format-specific function for MAJOR-MODE."
               (setq new-end (subed-timestamp-to-msecs (match-string 0))))
             (subed-jump-to-subtitle-text)
             (delete-region pos (point))
-            (insert "\n")
-            (subed-set-subtitle-time-stop new-end))
+            (insert " ")
+            (let ((subed-enforce-time-boundaries nil))
+              (subed-set-subtitle-time-stop new-end))
+            (run-hooks 'subed-subtitle-merged-hook))
         (error "No subtitle to merge into")))))
 
 ;;; Maintenance
@@ -376,8 +421,7 @@ Use the format-specific function for MAJOR-MODE."
     (atomic-change-group
       (let ((orig-point (point)))
         (goto-char (point-min))
-        (while (and (re-search-forward (format "^\\(%s\\)" subed--regexp-timestamp) nil t)
-                    (goto-char (match-beginning 1)))
+        (while (subed-forward-subtitle-id)
           ;; This regex is stricter than `subed--regexp-timestamp'
           (unless (looking-at "^\\([0-9]\\{2\\}:\\)?[0-9]\\{2\\}:[0-9]\\{2\\}\\(\\.[0-9]\\{0,3\\}\\)")
             (error "Found invalid start time: %S"  (substring (or (thing-at-point 'line :no-properties) "\n") 0 -1)))
