@@ -1,4 +1,4 @@
-;; -*- eval: (buttercup-minor-mode) -*-
+;; -*- lexical-binding: t; eval: (buttercup-minor-mode) -*-
 
 (load-file "./tests/undercover-init.el")
 (require 'subed)
@@ -24,7 +24,7 @@ Baz.
     (subed-vtt-mode)
     (progn ,@body)))
 
-(describe "VTT"
+(describe "subed-vtt"
   (describe "Getting"
     (describe "the subtitle ID"
       (it "returns the subtitle ID if it can be found."
@@ -34,7 +34,14 @@ Baz.
          (expect (subed-subtitle-id) :to-equal "00:01:01.000")))
       (it "returns nil if no subtitle ID can be found."
         (with-temp-vtt-buffer
-         (expect (subed-subtitle-id) :to-equal nil))))
+         (expect (subed-subtitle-id) :to-equal nil)))
+      (it "handles extra attributes"
+        (with-temp-vtt-buffer
+         (insert "WEBVTT
+
+00:00:01.000 --> 00:00:02.000 align:start position:0%
+Hello world")
+         (expect (subed-subtitle-id) :to-equal "00:00:01.000"))))
     (describe "the subtitle ID at playback time"
       (it "returns subtitle ID if time is equal to start time."
         (with-temp-vtt-buffer
@@ -165,8 +172,31 @@ Baz.
          (insert "foo")
          (expect (subed-subtitle-relative-point) :to-equal nil)))
       )
-    )
-
+    (describe "the subtitle start position"
+      (it "returns the start from inside a subtitle."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-data)
+         (re-search-backward "Bar")
+         (expect (subed-subtitle-start-pos) :to-equal 45)))
+      (it "returns the start from the beginning of the line."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-data)
+         (re-search-backward "00:02:02\\.234")
+         (expect (subed-subtitle-start-pos) :to-equal 45)))
+      (it "returns the start of a comment"
+        (with-temp-vtt-buffer
+         (insert mock-vtt-data)
+         (re-search-backward "00:02:02\\.234")
+         (insert "NOTE\n\nThis is a comment\n\n")
+         (expect (subed-subtitle-start-pos) :to-equal 45)))))
+  (describe "Converting to msecs"
+    (it "works with numbers."
+      (expect (with-temp-vtt-buffer (subed-to-msecs 5123)) :to-equal 5123))
+    (it "works with numbers as strings."
+      (expect (with-temp-vtt-buffer (subed-to-msecs "5123")) :to-equal 5123))
+    (it "works with timestamps."
+      (expect (with-temp-vtt-buffer
+               (subed-to-msecs "00:00:05.124")) :to-equal 5124)))
   (describe "Jumping"
     (describe "to current subtitle timestamp"
       (it "returns timestamp's point when point is already on the timestamp."
@@ -215,25 +245,227 @@ Baz.
          (expect (subed-jump-to-subtitle-time-start) :to-equal 9)
          (expect (looking-at subed--regexp-timestamp) :to-be t)
          (expect (match-string 0) :to-equal "00:01:01.000")))
-      )
-    (describe "to specific subtitle by timestamp"
-      (it "returns timestamp's point if wanted time exists."
+      (it "works with short timestamps from a comment."
         (with-temp-vtt-buffer
-         (insert mock-vtt-data)
-         (goto-char (point-max))
-         (expect (subed-jump-to-subtitle-id "00:02:02.234") :to-equal 45)
-         (expect (looking-at (regexp-quote "00:02:02.234")) :to-be t)
-         (expect (subed-jump-to-subtitle-id "00:01:01.000") :to-equal 9)
-         (expect (looking-at (regexp-quote "00:01:01.000")) :to-be t)))
-      (it "returns nil and does not move if wanted ID does not exists."
-        (with-temp-vtt-buffer
-         (insert mock-vtt-data)
-         (goto-char (point-min))
-         (search-forward "Foo")
-         (let ((stored-point (point)))
-           (expect (subed-jump-to-subtitle-id "0:08:00") :to-equal nil)
-           (expect stored-point :to-equal (point)))))
+         (insert "WEBVTT\n\nNOTE A comment goes here
+
+09:34.900 --> 00:09:37.659
+Subtitle 1
+
+00:10:34.900 --> 00:11:37.659
+Subtitle 2")
+         (re-search-backward "NOTE")
+         (goto-char (line-beginning-position))
+         (expect (subed-jump-to-subtitle-time-start) :to-equal 35)))
+
       )
+    (describe "to subtitle start pos"
+			(describe "in the current subtitle"
+				(it "returns nil in the header."
+					(with-temp-vtt-buffer
+					 (insert mock-vtt-data)
+					 (goto-char (point-min))
+					 (expect (subed-jump-to-subtitle-start-pos) :to-be nil)))
+				(it "goes to the ID if specified."
+					(with-temp-vtt-buffer
+					 (insert "WEBVTT
+
+1
+00:01:01.000 --> 00:01:05.123
+Foo.
+
+00:02:02.234 --> 00:02:10.345
+Bar.
+")
+					 (re-search-backward "Foo")
+					 (expect (subed-jump-to-subtitle-start-pos) :not :to-be nil)
+					 (expect (looking-at "1") :to-be t)))
+				(it "goes to the timestamp if there is no ID."
+					(with-temp-vtt-buffer
+					 (insert "WEBVTT
+
+1
+00:01:01.000 --> 00:01:05.123
+Foo.
+
+00:02:02.234 --> 00:02:10.345
+Bar.
+")
+					 (re-search-backward "Bar")
+					 (expect (subed-jump-to-subtitle-start-pos) :not :to-be nil)
+					 (expect (looking-at "00:02:02.234") :to-be t)))
+        (it "goes to the comment if there is one."
+					(with-temp-vtt-buffer
+					 (insert "WEBVTT
+
+NOTE This is a comment
+
+1
+00:01:01.000 --> 00:01:05.123
+Foo.
+
+00:02:02.234 --> 00:02:10.345
+Bar.
+")
+					 (re-search-backward "Foo")
+					 (expect (subed-jump-to-subtitle-start-pos) :not :to-be nil)
+					 (expect (looking-at "NOTE This is a comment") :to-be t)))
+				(describe "when called from a comment"
+					(it "goes to the start of the comment."
+						(with-temp-vtt-buffer
+						 (insert "WEBVTT
+
+NOTE
+This is a comment
+
+1
+00:01:01.000 --> 00:01:05.123
+Foo.
+
+00:02:02.234 --> 00:02:10.345
+Bar.
+")
+						 (re-search-backward "This is a comment")
+						 (expect (subed-jump-to-subtitle-start-pos) :not :to-be nil)
+						 (expect (looking-at "NOTE\nThis is a comment") :to-be t))))))
+    (describe "to subtitle ID"
+      (describe "in the current subtitle"
+        (it "returns nil in the header."
+          (with-temp-vtt-buffer
+           (insert mock-vtt-data)
+           (goto-char (point-min))
+           (expect (subed-jump-to-subtitle-id) :to-be nil)))
+        (it "goes to the ID if specified."
+          (with-temp-vtt-buffer
+           (insert "WEBVTT
+
+1
+00:01:01.000 --> 00:01:05.123
+Foo.
+
+00:02:02.234 --> 00:02:10.345
+Bar.
+")
+           (re-search-backward "Foo")
+           (expect (subed-jump-to-subtitle-id) :not :to-be nil)
+           (expect (looking-at "1") :to-be t)))
+        (it "goes to the timestamp if there is no ID."
+          (with-temp-vtt-buffer
+           (insert "WEBVTT
+
+1
+00:01:01.000 --> 00:01:05.123
+Foo.
+
+00:02:02.234 --> 00:02:10.345
+Bar.
+")
+           (re-search-backward "Bar")
+           (expect (subed-jump-to-subtitle-id) :not :to-be nil)
+           (expect (looking-at "00:02:02.234") :to-be t)))
+        (describe "when called from a comment"
+          (it "goes to the ID of the subtitle after the comment."
+					  (with-temp-vtt-buffer
+					   (insert "WEBVTT
+
+NOTE
+This is a comment
+
+1
+00:01:01.000 --> 00:01:05.123
+Foo.
+
+00:02:02.234 --> 00:02:10.345
+Bar.
+")
+					   (re-search-backward "This is a comment")
+					   (expect (subed-jump-to-subtitle-id) :not :to-be nil)
+					   (expect (looking-at "1\n") :to-be t)))
+          (it "goes to the timestamp of the subtitle after the comment if no ID is specified."
+					  (with-temp-vtt-buffer
+					   (insert "WEBVTT
+
+1
+00:01:01.000 --> 00:01:05.123
+Foo.
+
+NOTE
+This is a comment
+
+00:02:02.234 --> 00:02:10.345
+Bar.
+
+00:03:02.234 --> 00:03:10.345
+Baz.
+")
+					   (re-search-backward "This is a comment")
+					   (expect (subed-jump-to-subtitle-id) :not :to-be nil)
+					   (expect (looking-at (regexp-quote "00:02:02.234")) :to-be t)))
+          (it "goes to the timestamp of the subtitle after the comment even with a short timestamp."
+					  (with-temp-vtt-buffer
+					   (insert "WEBVTT
+
+1
+00:01:01.000 --> 00:01:05.123
+Foo.
+
+NOTE
+This is a comment
+
+02:02.234 --> 00:02:10.345
+Bar.
+
+00:03:02.234 --> 00:03:10.345
+Baz.
+")
+					   (re-search-backward "This is a comment")
+					   (expect (subed-jump-to-subtitle-id) :not :to-be nil)
+					   (expect (looking-at (regexp-quote "02:02.234")) :to-be t)))
+          ))
+      (describe "when given an ID"
+        (it "returns ID's point if wanted time exists."
+          (with-temp-vtt-buffer
+           (insert "WEBVTT
+
+1
+00:01:01.000 --> 00:01:05.123
+Foo.
+
+NOTE
+This is a comment
+
+00:02:02.234 --> 00:02:10.345
+Bar.
+")
+           (goto-char (point-max))
+           (expect (subed-jump-to-subtitle-id "1") :not :to-be nil)
+           (expect (looking-at "1\n") :to-be t)))
+        (it "returns nil and does not move if wanted ID does not exists."
+          (with-temp-vtt-buffer
+           (insert mock-vtt-data)
+           (goto-char (point-min))
+           (search-forward "Foo")
+           (let ((stored-point (point)))
+             (expect (subed-jump-to-subtitle-id "3") :to-equal nil)
+             (expect stored-point :to-equal (point))))))
+      (describe "when given a timestamp"
+        (it "returns timestamp's point if wanted time exists."
+          (with-temp-vtt-buffer
+           (insert mock-vtt-data)
+           (goto-char (point-max))
+           (expect (subed-jump-to-subtitle-id "00:02:02.234") :to-equal 45)
+           (expect (looking-at (regexp-quote "00:02:02.234")) :to-be t)
+           (expect (subed-jump-to-subtitle-id "00:01:01.000") :to-equal 9)
+           (expect (looking-at (regexp-quote "00:01:01.000")) :to-be t)))
+        (it "returns nil and does not move if wanted ID does not exists."
+          (with-temp-vtt-buffer
+           (insert mock-vtt-data)
+           (goto-char (point-min))
+           (search-forward "Foo")
+           (let ((stored-point (point)))
+             (expect (subed-jump-to-subtitle-id "0:08:00") :to-equal nil)
+             (expect stored-point :to-equal (point)))))))
+
     (describe "to subtitle start time"
       (it "returns start time's point if movement was successful."
         (with-temp-vtt-buffer
@@ -293,6 +525,18 @@ Baz.
       (it "returns nil if movement failed."
         (with-temp-vtt-buffer
          (expect (subed-jump-to-subtitle-time-stop) :to-equal nil)))
+      (it "works with short timestamps from a comment."
+        (with-temp-vtt-buffer
+         (insert "WEBVTT\n\nNOTE A comment goes here
+
+09:34.900 --> 00:09:37.659
+Subtitle 1
+
+00:10:34.900 --> 00:11:37.659
+Subtitle 2")
+         (re-search-backward "NOTE")
+         (goto-char (line-beginning-position))
+         (expect (subed-jump-to-subtitle-text) :to-equal 62)))
       )
     (describe "to end of subtitle text"
       (it "returns point if subtitle end can be found."
@@ -364,7 +608,33 @@ Baz.
          (backward-char)
          (expect (subed-jump-to-subtitle-end) :to-be 98)
          (expect (looking-at "^$") :to-be t)))
-      )
+      (it "works with short timestamps from a comment."
+        (with-temp-vtt-buffer
+         (insert "WEBVTT\n\nNOTE A comment goes here
+
+09:34.900 --> 00:09:37.659
+Subtitle 1
+
+00:10:34.900 --> 00:11:37.659
+Subtitle 2")
+         (re-search-backward "NOTE")
+         (goto-char (line-beginning-position))
+         (expect (subed-jump-to-subtitle-end) :to-equal 72)))
+      (it "works with optional IDs and multi-line cues where a line is all numbers."
+        (with-temp-vtt-buffer
+         (insert "WEBVTT
+
+1
+00:00:00.000 --> 00:00:01.000
+This is first subtitle.
+123456789
+
+2
+00:00:01.000 --> 00:00:0.,000
+This is second subtitle.
+")
+         (re-search-backward "This is first")
+         (expect (subed-jump-to-subtitle-end) :to-be 74))))
     (describe "to next subtitle ID"
       (it "returns point when there is a next subtitle."
         (with-temp-vtt-buffer
@@ -418,6 +688,10 @@ Baz.
          (expect (looking-at (regexp-quote "00:03:15.5")) :to-be t)
          (expect (subed-backward-subtitle-id) :to-be 45)
          (expect (looking-at (regexp-quote "00:02:02.234")) :to-be t)))
+      (it "does not get confused by empty lines at the end of the buffer."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-data "\n\n")
+         (expect (subed-backward-subtitle-id) :not :to-be nil)))
       (it "returns nil and doesn't move when there is no previous subtitle."
         (with-temp-vtt-buffer
          (expect (subed-backward-subtitle-id) :to-be nil))
@@ -571,15 +845,14 @@ Baz.
          (expect (looking-at (regexp-quote "00:01:01.000")) :to-be t)
          (expect (subed-backward-subtitle-time-stop) :to-be nil)
          (expect (looking-at (regexp-quote "00:01:01.000")) :to-be t)))
-      )
-    )
+      ))
 
   (describe "Setting start/stop time"
-    (it "of current subtitle."
+    (it "of current subtitle updates it."
       (with-temp-vtt-buffer
        (insert mock-vtt-data)
        (subed-jump-to-subtitle-end "00:02:02.234")
-       (subed-set-subtitle-time-start (+ (* 1 60 60 1000) (* 2 60 1000) (* 3 1000) 400))
+       (subed-set-subtitle-time-start (+ (* 1 60 60 1000) (* 2 60 1000) (* 3 1000) 400) nil t t)
        (expect (buffer-string) :to-equal (concat "WEBVTT\n\n"
                                                  "00:01:01.000 --> 00:01:05.123\n"
                                                  "Foo.\n\n"
@@ -587,7 +860,7 @@ Baz.
                                                  "Bar.\n\n"
                                                  "00:03:03.45 --> 00:03:15.5\n"
                                                  "Baz.\n"))
-       (subed-set-subtitle-time-stop (+ (* 5 60 60 1000) (* 6 60 1000) (* 7 1000) 800))
+       (subed-set-subtitle-time-stop (+ (* 5 60 60 1000) (* 6 60 1000) (* 7 1000) 800) nil t t)
        (expect (buffer-string) :to-equal (concat "WEBVTT\n\n"
                                                  "00:01:01.000 --> 00:01:05.123\n"
                                                  "Foo.\n\n"
@@ -595,11 +868,11 @@ Baz.
                                                  "Bar.\n\n"
                                                  "00:03:03.45 --> 00:03:15.5\n"
                                                  "Baz.\n"))))
-    (it "of specific subtitle."
+    (it "of specific subtitle updates it."
       (with-temp-vtt-buffer
        (insert mock-vtt-data)
        (subed-jump-to-subtitle-time-stop "00:01:01.000")
-       (subed-set-subtitle-time-start (+ (* 2 60 60 1000) (* 4 60 1000) (* 6 1000) 800) 1)
+       (subed-set-subtitle-time-start (+ (* 2 60 60 1000) (* 4 60 1000) (* 6 1000) 800) 1 t t)
        (expect (buffer-string) :to-equal (concat "WEBVTT\n\n"
                                                  "02:04:06.800 --> 00:01:05.123\n"
                                                  "Foo.\n\n"
@@ -608,7 +881,7 @@ Baz.
                                                  "00:03:03.45 --> 00:03:15.5\n"
                                                  "Baz.\n"))
        (subed-jump-to-subtitle-text "00:03:03.45")
-       (subed-set-subtitle-time-stop (+ (* 3 60 60 1000) (* 5 60 1000) (* 7 1000) 900) 3)
+       (subed-set-subtitle-time-stop (+ (* 3 60 60 1000) (* 5 60 1000) (* 7 1000) 900) 3 t t)
        (expect (buffer-string) :to-equal (concat "WEBVTT\n\n"
                                                  "02:04:06.800 --> 00:01:05.123\n"
                                                  "Foo.\n\n"
@@ -616,18 +889,37 @@ Baz.
                                                  "Bar.\n\n"
                                                  "00:03:03.45 --> 03:05:07.900\n"
                                                  "Baz.\n"))))
-    (it "when milliseconds lack digits."
+    (it "when milliseconds lack digits, fills the rest in."
       (with-temp-vtt-buffer
        (insert mock-vtt-data)
        (subed-jump-to-subtitle-id "00:03:03.45")
-       (subed-set-subtitle-time-start (+ (* 1 60 60 1000) (* 2 60 1000) (* 3 1000) 4) 3)
+       (subed-set-subtitle-time-start (+ (* 1 60 60 1000) (* 2 60 1000) (* 3 1000) 4) 3 t t)
        (expect (save-excursion (subed-jump-to-subtitle-time-start)
                                (thing-at-point 'line))
                :to-equal "01:02:03.004 --> 00:03:15.5\n")
-       (subed-set-subtitle-time-stop (+ (* 2 60 60 1000) (* 3 60 1000) (* 4 1000) 60) 3)
+       (subed-set-subtitle-time-stop (+ (* 2 60 60 1000) (* 3 60 1000) (* 4 1000) 60) 3 t t)
        (expect (save-excursion (subed-jump-to-subtitle-time-start)
                                (thing-at-point 'line))
                :to-equal "01:02:03.004 --> 02:03:04.060\n"))))
+  (describe "Shifting subtitles"
+    (describe "starting at a specific timestamp"
+      (it "works when called from the start of the buffer."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-data)
+         (goto-char (point-min))
+         (subed-shift-subtitles-to-start-at-timestamp 500)
+         (let ((data (subed-subtitle-list)))
+           (expect (elt (elt (subed-subtitle-list) 0) 1) :to-equal 500)
+           (expect (elt (elt (subed-subtitle-list) 0) 2) :to-equal 4623)
+           (expect (elt (elt (subed-subtitle-list) 2) 1) :to-equal 122950))))
+      (it "only affects the current and following subtitles."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-data)
+         (re-search-backward "^Bar")
+         (subed-shift-subtitles-to-start-at-timestamp 120000)
+         (let ((data (subed-subtitle-list)))
+           (expect (elt (elt (subed-subtitle-list) 0) 1) :to-equal 61000)
+           (expect (elt (elt (subed-subtitle-list) 1) 1) :to-equal 120000))))))
 
   (describe "Inserting a subtitle"
     (describe "in an empty buffer"
@@ -654,30 +946,40 @@ Baz.
                                                      "Foo. bar\nbaz.\n"))
            (expect (point) :to-equal 31)))
         )
-      (describe "after"
-        (it "passing nothing."
+      (describe "when appending"
+        (it "creates a subtitle with default arguments."
           (with-temp-vtt-buffer
            (expect (subed-append-subtitle) :to-equal 31)
            (expect (buffer-string) :to-equal (concat "00:00:00.000 --> 00:00:01.000\n\n"))
            (expect (point) :to-equal 31)))
-        (it "passing start time."
+        (it "creates a subtitle with a start time."
           (with-temp-vtt-buffer
            (expect (subed-append-subtitle nil 60000) :to-equal 31)
            (expect (buffer-string) :to-equal (concat "00:01:00.000 --> 00:01:01.000\n\n"))
            (expect (point) :to-equal 31)))
-        (it "passing start time and stop time."
+        (it "creates a subtitle with a start time and stop time."
           (with-temp-vtt-buffer
            (expect (subed-append-subtitle nil 60000 65000) :to-equal 31)
            (expect (buffer-string) :to-equal (concat "00:01:00.000 --> 00:01:05.000\n\n"))
            (expect (point) :to-equal 31)))
-        (it "passing start time, stop time and text."
+        (it "creates a subtitle with a start time, stop time and text."
           (with-temp-vtt-buffer
            (expect (subed-append-subtitle nil 60000 65000 "Foo, bar\nbaz.") :to-equal 31)
            (expect (buffer-string) :to-equal (concat "00:01:00.000 --> 00:01:05.000\n"
                                                      "Foo, bar\nbaz.\n"))
            (expect (point) :to-equal 31)))
-        )
-      )
+        (it "creates a subtitle with a start time, stop time, text, and a single-line comment."
+          (with-temp-vtt-buffer
+           (subed-append-subtitle nil 60000 65000 "Foo, bar\nbaz." "Hello")
+           (expect (buffer-string) :to-equal (concat "NOTE Hello\n\n00:01:00.000 --> 00:01:05.000\n"
+                                                     "Foo, bar\nbaz.\n"))
+           (expect (looking-at "Foo") :to-be t)))
+        (it "creates a subtitle with a start time, stop time, text, and a multi-line comment."
+          (with-temp-vtt-buffer
+           (subed-append-subtitle nil 60000 65000 "Foo, bar\nbaz." "Hello\nworld")
+           (expect (buffer-string) :to-equal (concat "NOTE\nHello\nworld\n\n00:01:00.000 --> 00:01:05.000\n"
+                                                     "Foo, bar\nbaz.\n"))
+           (expect (looking-at "Foo") :to-be t)))))
     (describe "in a non-empty buffer"
       (describe "before the current subtitle"
         (describe "with point on the first subtitle"
@@ -837,14 +1139,15 @@ Baz.
              (expect (point) :to-equal 67)))
           )
         (describe "with point on a non-last subtitle"
-          (it "passing nothing."
+          (it "inserts an empty subtitle."
             (with-temp-vtt-buffer
              (insert (concat "00:00:01.000 --> 00:00:02.000\n"
                              "Foo.\n\n"
                              "00:00:05.000 --> 00:00:06.000\n"
                              "Bar.\n"))
              (subed-jump-to-subtitle-time-start "00:00:01.000")
-             (expect (subed-append-subtitle) :to-equal 67)
+             (subed-append-subtitle)
+             (expect (point) :to-equal 67)
              (expect (buffer-string) :to-equal (concat "00:00:01.000 --> 00:00:02.000\n"
                                                        "Foo.\n\n"
                                                        "00:00:00.000 --> 00:00:01.000\n"
@@ -899,6 +1202,24 @@ Baz.
              (expect (point) :to-equal 67)))
           )
         )
+      (describe "before a comment"
+        (it "inserts before the comment."
+            (with-temp-vtt-buffer
+             (insert (concat "00:00:01.000 --> 00:00:02.000\n"
+                             "Foo.\n\n"
+                             "NOTE comment\n\n00:00:05.000 --> 00:00:06.000\n"
+                             "Bar.\n"))
+             (subed-jump-to-subtitle-time-start "00:00:01.000")
+             (expect (subed-append-subtitle nil 2500 4000 "Baz.") :to-equal 67)
+             (expect (buffer-string) :to-equal (concat "00:00:01.000 --> 00:00:02.000\n"
+                                                       "Foo.\n\n"
+                                                       "00:00:02.500 --> 00:00:04.000\n"
+                                                       "Baz.\n\n"
+                                                       "NOTE comment\n\n00:00:05.000 --> 00:00:06.000\n"
+                                                       "Bar.\n"))
+             (expect (point) :to-equal 67))
+          )
+        )
       (it "when point is on empty text."
         (with-temp-vtt-buffer
          (insert (concat "00:00:01.000 --> 00:00:02.000\n"
@@ -1040,6 +1361,12 @@ Baz.
        (forward-char 2)
        (subed-validate)
        (expect (point) :to-equal 77)))
+    (it "accepts cue text that starts with something that looks like a timestamp."
+      (with-temp-vtt-buffer
+       (insert "WebVTT\n\n00:00:00.003 --> 00:00:05.123\n12:00 is noon.\n\n00:10:00.003 --> 00:11:05.123\nThis should be fine.")
+       (subed-validate)
+       (expect (point) :to-equal (point-max))))
+
     )
 
   (describe "Sanitizing"
@@ -1242,8 +1569,8 @@ Baz.
                 "\n"
                 "00:12:01.000 --> 00:01:05.123\n"
                 "Foo.\n"))))
-    (describe "preserves point in the current subtitle"
-      (it "when subtitle text is non-empty."
+    (describe "point preservation"
+      (it "works when subtitle text is non-empty."
         (with-temp-vtt-buffer
          (insert mock-vtt-data)
          (goto-char (point-min))
@@ -1253,23 +1580,27 @@ Baz.
          (expect (current-word) :to-equal "Foo")
          (subed-sort)
          (expect (current-word) :to-equal "Foo")))
-      (it "when subtitle text is empty."
+      (it "works when subtitle text is empty."
         (with-temp-vtt-buffer
          (insert "WEBVTT\n\n00:12:01.000 --> 00:01:05.123\n")
          (let ((pos (point)))
            (subed-sort)
            (expect (buffer-string) :to-equal "WEBVTT\n\n00:12:01.000 --> 00:01:05.123\n\n")
            (expect (point) :to-equal pos))))
-      )
-    )
+      (it "works in the header."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-data)
+         (goto-char (point-min))
+         (subed-sort)
+         (expect (point) :to-equal (point-min))))))
   (describe "Converting msecs to timestamp"
     (it "uses the right format"
       (with-temp-vtt-buffer
        (expect (subed-msecs-to-timestamp 1401) :to-equal "00:00:01.401"))))
   (describe "Working with comments"
-    (it "ignores the comment when jumping to the end of the subtitle"
-      (with-temp-vtt-buffer
-       (insert "WEBVTT
+    (before-each
+      (setq mock-vtt-comments-data
+            "WEBVTT
 
 00:00:00.000 --> 00:00:01.000
 This is a test.
@@ -1279,13 +1610,140 @@ and have more text as needed.
 
 00:01:00.000 --> 00:00:02.000
 This is another test here.
-")
+"))
+    (it "ignores the comment when jumping to the end of the subtitle"
+      (with-temp-vtt-buffer
+       (insert mock-vtt-comments-data)
        (goto-char (point-min))
        (subed-forward-subtitle-end)
        (expect (current-word) :to-equal "test")
        (subed-forward-subtitle-end)
-       (expect (current-word) :to-equal "here"))))
+       (expect (current-word) :to-equal "here")))
+    (describe "jumping to the comment"
+      (it "returns nil when there is no comment."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (re-search-backward "This is a test")
+         (expect (subed-jump-to-subtitle-comment) :to-be nil)))
+      (it "jumps to the comment for the current subtitle."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (goto-char (point-max))
+         (subed-jump-to-subtitle-comment)
+         (expect (looking-at "NOTE A comment") :to-be t))))
+    (describe "getting the comment"
+      (it "returns nil when there is no comment."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (re-search-backward "This is a test")
+         (expect (subed-subtitle-comment) :to-be nil)))
+      (it "returns the comment."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (goto-char (point-max))
+         (expect (subed-subtitle-comment) :to-equal "A comment can go here\nand have more text as needed."))))
+    (describe "setting the comment"
+      (it "sets the comment when there isn't one yet."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (re-search-backward "This is a test")
+         (subed-set-subtitle-comment "Skip")
+         (expect (buffer-string) :to-equal
+                 "WEBVTT
 
+NOTE Skip
+
+00:00:00.000 --> 00:00:01.000
+This is a test.
+
+NOTE A comment can go here
+and have more text as needed.
+
+00:01:00.000 --> 00:00:02.000
+This is another test here.
+")))
+      (it "replaces the comment."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (goto-char (point-max))
+         (subed-set-subtitle-comment "Replaced.")
+         (expect (buffer-string) :to-equal
+                 "WEBVTT
+
+00:00:00.000 --> 00:00:01.000
+This is a test.
+
+NOTE Replaced.
+
+00:01:00.000 --> 00:00:02.000
+This is another test here.
+")))
+      (it "clears the comment."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (goto-char (point-max))
+         (subed-set-subtitle-comment nil)
+         (expect (buffer-string) :to-equal
+                 "WEBVTT
+
+00:00:00.000 --> 00:00:01.000
+This is a test.
+
+00:01:00.000 --> 00:00:02.000
+This is another test here.
+"))))
+    (describe "going to the next subtitle's comment"
+      (it "returns nil in an empty buffer."
+        (with-temp-vtt-buffer
+         (expect (subed-forward-subtitle-comment) :to-be nil)))
+      (it "returns nil at the end of the file."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (expect (subed-forward-subtitle-comment) :to-be nil)))
+      (it "returns nil if the next subtitle does not have a comment."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (save-excursion (subed-append-subtitle))
+         (expect (subed-forward-subtitle-comment) :to-be nil)))
+      (it "jumps to the next subtitle's comment."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (re-search-backward "This is a test")
+         (expect (subed-forward-subtitle-comment) :not :to-be nil)
+         (expect (looking-at "NOTE ") :to-be t))))
+    (describe "going to the previous comment"
+      (it "returns nil in an empty buffer."
+        (with-temp-vtt-buffer
+         (expect (subed-backward-subtitle-comment) :to-be nil)))
+      (it "returns nil at the start of the file."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (goto-char (point-min))
+         (expect (subed-backward-subtitle-comment) :to-be nil)))
+      (it "returns nil if the previous subtitle does not have a comment."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (re-search-backward "This is another test here")
+         (expect (subed-backward-subtitle-comment) :to-be nil)))
+      (it "jumps to the previous subtitle's comment."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-comments-data)
+         (subed-append-subtitle)
+         (expect (subed-backward-subtitle-comment) :not :to-be nil)
+         (expect (looking-at "NOTE ") :to-be t))))
+    (describe "when the cue text starts with Note"
+      (it "is not confused."
+        (with-temp-vtt-buffer
+         (insert "WEBVTT
+
+00:00:00.000 --> 00:00:00.999
+Note this is a test
+
+00:00:01.000 --> 00:00:01.000
+another test
+")
+         (let ((case-fold-search t))
+           (expect (elt (car (subed-subtitle-list)) 3) :to-equal "Note this is a test"))))))
   (describe "Merging with next subtitle"
     (it "throws an error in an empty buffer."
       (with-temp-vtt-buffer
@@ -1300,9 +1758,21 @@ This is another test here.
        (insert mock-vtt-data)
        (subed-jump-to-subtitle-text "00:02:02.234")
        (subed-merge-with-next)
-       (expect (subed-subtitle-text) :to-equal "Bar.\nBaz.")
+       (expect (subed-subtitle-text) :to-equal "Bar. Baz.")
        (expect (subed-subtitle-msecs-start) :to-equal 122234)
-       (expect (subed-subtitle-msecs-stop) :to-equal 195500))))
+       (expect (subed-subtitle-msecs-stop) :to-equal 195500)))
+		(it "updates looping."
+			(with-temp-vtt-buffer
+       (insert mock-vtt-data)
+       (subed-jump-to-subtitle-text "00:02:02.234")
+			 (let ((subed-loop-seconds-before 1)
+						 (subed-loop-seconds-after 1))
+				 (subed--set-subtitle-loop)
+				 (expect subed--subtitle-loop-start :to-equal (subed-timestamp-to-msecs "00:02:01.234"))
+				 (expect subed--subtitle-loop-stop :to-equal (subed-timestamp-to-msecs "00:02:11.345"))
+				 (subed-merge-with-next)
+				 (expect subed--subtitle-loop-start :to-equal (subed-timestamp-to-msecs "00:02:01.234"))
+				 (expect subed--subtitle-loop-stop :to-equal (subed-timestamp-to-msecs "00:03:16.500"))))))
   (describe "Font-locking"
     (it "recognizes VTT syntax."
       (with-temp-vtt-buffer
@@ -1355,4 +1825,28 @@ World
 00:01:21.058 --> 00:01:23.868
 Again")
        (with-current-buffer (subed-convert "TXT" t)
-         (expect (buffer-string) :to-equal "Hello\n\nNOTE Comment\n\nWorld\nAgain\n"))))))
+         (expect (buffer-string) :to-equal "Hello\n\nComment\n\nWorld\nAgain\n")))))
+  (describe "iterating over subtitles"
+    (describe "forwards"
+      (it "handles headers."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-data)
+         (let (result)
+           (subed-for-each-subtitle (point-min) (point-max) nil
+             (add-to-list 'result (point)))
+           (expect (length result) :to-equal 3)))))
+    (describe "backwards"
+      (it "handles headers."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-data)
+         (let (result)
+           (subed-for-each-subtitle (point-min) (point-max) t
+             (add-to-list 'result (point)))
+           (expect (length result) :to-equal 3))))
+      (it "handles empty lines."
+        (with-temp-vtt-buffer
+         (insert mock-vtt-data "\n\n")
+         (let (result)
+           (subed-for-each-subtitle (point-min) (point-max) t
+             (add-to-list 'result (point)))
+           (expect (length result) :to-equal 3)))))))
